//////////////////////////////////////
// author : Dirk Duschinger         //
// email  : dirk.duschigner@cern.ch //
//////////////////////////////////////

#include "CursesInputHandler.h"

CursesInputHandler::CursesInputHandler()
  : m_coh(0)
  , m_hBufferHist(0)
  , m_iCursorX(0)
  , m_iCursorY(0)
{
}

CursesInputHandler::~CursesInputHandler()
{
}

void CursesInputHandler::initialize(CursesOutputHandler* coh)
{
  m_coh = coh;
  keypad(stdscr, true);
}

void CursesInputHandler::Cycle()
{
  int key = 0;
  move(m_iCursorX,m_iCursorY);
  m_coh->WritePanel();
  m_coh->UpdatePanelSelection(0);
  m_coh->UpdateScreen();

  while (key != 'q')
  {
    key = getch();
    if (key == 'r')
      Resize();
    if (key == 's')
      Save();
    if (key == 'a')
      Add();
    if (key == KEY_PPAGE)
      for(unsigned short i=0; i<10; i++)
        ScrollUp();
    if (key == KEY_NPAGE)
      for(unsigned short i=0; i<10; i++)
        ScrollDown();
    if (key == KEY_DOWN)
      ScrollDown();
    if (key == KEY_UP)
      ScrollUp();
    if (key == KEY_LEFT)
      MoveSeperatorLeft();
    if (key == KEY_RIGHT)
      MoveSeperatorRight();
    if (key == 10)		// 10=Enter
      PanelEntryInteraction();
    if (key == 'h')
      Help();
    m_coh->UpdatePanelSelection(m_iCursorY);
    m_coh->UpdateScreen();
  }
}

void CursesInputHandler::Add()
{
  ROOTPanelEntry* rpe = m_coh->GetPanelEntry(m_iCursorY-m_coh->GetOffsetLeftPanel());
  if (m_hBufferHist!=NULL)
  {
    m_hBufferHist->Add(dynamic_cast<TH1*>(rpe->GetObject()));
    TCanvas* can = new TCanvas();
    m_hBufferHist->Draw();
    can->SaveAs("/tmp/eps.eps");
    delete can;

    EpsParser* ep = new EpsParser("/tmp/eps.eps");
    ep->Draw(m_coh->GetPadWindow());
    delete m_hBufferHist;
    m_hBufferHist = NULL;
  }
  else
  {
    if (rpe->GetObject()->InheritsFrom("TH1"))
    {
      m_hBufferHist = dynamic_cast<TH1*>(rpe->GetObject());
      // continue;
    }
  }
}

void CursesInputHandler::Help()
{
  m_coh->OpenHelp();
  getch();
  m_coh->CloseHelp();
}

void CursesInputHandler::MoveSeperatorLeft()
{
  int newwidth = m_coh->GetWidthLeftPanel()-1;
  m_coh->SetWidthLeftPanel((newwidth > 2) ? newwidth : 2);
  m_coh->reinitialize();
  m_coh->DrawFrame();
  m_coh->UpdateScreen();
}

void CursesInputHandler::MoveSeperatorRight()
{
  int newwidth = m_coh->GetWidthLeftPanel()+1;
  m_coh->SetWidthLeftPanel((newwidth < m_coh->GetCursesXRes()-2) ? newwidth:m_coh->GetCursesXRes()-2);
  m_coh->reinitialize();
  m_coh->DrawFrame();
  m_coh->WritePanel();
  m_coh->UpdateScreen();
}

void CursesInputHandler::PanelEntryInteraction()
{
  ROOTPanelEntry* rpe = m_coh->GetPanelEntry(m_iCursorY-m_coh->GetOffsetLeftPanel());
  if (!rpe) return;

  rpe->ToggleFold();
  if (!rpe->GetObject())
  {
    m_coh->WritePanel();
    return;
  }
  if (rpe->GetObject()->InheritsFrom("TDirectory") or rpe->GetObject()->InheritsFrom("TTree"))
  {
    m_coh->WritePanel();
    return;
  }

  TCanvas* can = new TCanvas();
  if (rpe->GetObject()->InheritsFrom("TH1"))
  {
    if (rpe->GetObject()->InheritsFrom("TH2"))
      rpe->GetObject()->Draw("colztext");
    else
      rpe->GetObject()->Draw();
    can->Update();
    can->SaveAs("/tmp/eps.eps");
    delete can;
  }
  else if (rpe->GetObject()->InheritsFrom("TBranch"))
  {
    rpe->GetParent()->GetObject()->Draw(rpe->GetName().c_str());
    can->Update();
    can->SaveAs("/tmp/eps.eps");
    delete can;
  }
  else if (rpe->GetObject()->InheritsFrom("TCanvas"))
  {
    rpe->GetObject()->SaveAs("/tmp/eps.eps");
  }
  EpsParser* ep = new EpsParser("/tmp/eps.eps");
  werase(m_coh->GetPadWindow());
  ep->Draw(m_coh->GetPadWindow());
  wrefresh(m_coh->GetPadWindow());

  m_coh->WritePanel();
}

void CursesInputHandler::Resize()
{
  struct winsize w;
  ioctl(0, TIOCGWINSZ, &w);
  werase(stdscr);
  werase(m_coh->GetPanelWindow());
  werase(m_coh->GetPadWindow());
  wresize(stdscr,w.ws_row,w.ws_col);
  m_coh->initialize();
  m_coh->DrawFrame();
  m_coh->WritePanel();
  m_coh->UpdateScreen();
}

void CursesInputHandler::Save()
{
  ROOTPanelEntry* rpe = m_coh->GetPanelEntry(m_iCursorY-m_coh->GetOffsetLeftPanel());
  char* c = m_coh->OpenSave();
  rpe->GetObject()->SaveAs(c);
  delete c;
  m_coh->CloseSave();
}

void CursesInputHandler::ScrollDown()
{
  m_iCursorY++;
  if (m_iCursorY == *(m_coh->GetPanelHeight()))
  {
    m_iCursorY--;
    int offset = m_coh->GetOffsetLeftPanel();
    offset--;
    offset = !(offset<0)?0:offset;
    m_coh->SetOffsetLeftPanel(offset);
    m_coh->WritePanel();
  }
  m_coh->UpdateScreen();
}

void CursesInputHandler::ScrollUp()
{
  m_iCursorY--;
  if (m_iCursorY == -1)
  {
    m_iCursorY++;
    int offset = m_coh->GetOffsetLeftPanel();
    offset++;
    offset = !(offset<0)?0:offset;
    m_coh->SetOffsetLeftPanel(offset);
    m_coh->WritePanel();
  }
  m_coh->UpdateScreen();
}

