//////////////////////////////////////
// author : Dirk Duschinger         //
// email  : dirk.duschigner@cern.ch //
//////////////////////////////////////

#include "EpsCommands.h"

EpsCommands::EpsCommands()
{
}

EpsCommands::EpsCommands(std::string com, std::vector<EpsPolygon*>* ListOfObjects, std::list<std::string>* sStack, std::vector<double>* dStack)
{
  if (com == "show") this->Show(ListOfObjects,sStack,dStack);
  if (com == "neg") this->Neg(dStack);
  if (com == "exch") this->Exch(dStack);
  if (com == "dup") this->Dup(dStack);
  if (com == "moveto") this->MoveTo(ListOfObjects,dStack);
  if (com == "rlineto") this->RLineTo(ListOfObjects,dStack);
  if (com == "closepath") this->ClosePath(ListOfObjects);
  if (com == "stroke") this->Stroke(ListOfObjects);
  if (com == "setrgbcolor") this->SetRGBColor(ListOfObjects,dStack);
  if (com == "setgray") this->SetGray(ListOfObjects,dStack);
  if (com == "eofill") this->EOFill(ListOfObjects);
}

EpsCommands::~EpsCommands()
{
}

void EpsCommands::Show(std::vector<EpsPolygon*>* ListOfObjects, std::list<std::string>* sStack, std::vector<double>* dStack)
{
  (*dStack).pop_back();
  double r = (*dStack).back();
  (*dStack).pop_back();
  double y = (*dStack).back();
  (*dStack).pop_back();
  double x = (*dStack).back();
  (*dStack).pop_back();
  std::string s = (*sStack).back();

  if (ListOfObjects->size()>0)
  {
    EpsPolygon* epsp;
    if (ListOfObjects->back()->GetCoordinateList().size() > 0)
    {
      epsp = new EpsPolygon();
      epsp->AddCoordinate(x,y);
      epsp->SetTitle(s);
      epsp->SetRotation(r);
      ListOfObjects->push_back(epsp);
    }
    else
    {
      epsp = (*ListOfObjects).back();
      epsp->AddCoordinate(x,y);
      epsp->SetRotation(r);
      epsp->SetTitle(s);
    }
  }
}

void EpsCommands::Neg(std::vector<double>* dStack)
{
  double val = (*dStack).back();
  (*dStack).pop_back();
  (*dStack).push_back(-val);
}

void EpsCommands::Exch(std::vector<double>* dStack)
{
  double val1 = (*dStack).back();
  (*dStack).pop_back();
  double val2 = (*dStack).back();
  (*dStack).pop_back();
  (*dStack).push_back(val1);
  (*dStack).push_back(val2);
}

void EpsCommands::Dup(std::vector<double>* dStack)
{
  double val = (*dStack).back();
  (*dStack).push_back(val);
}

void EpsCommands::MoveTo(std::vector<EpsPolygon*>* ListOfObjects,std::vector<double>* dStack)
{
  double y = (*dStack).back();
  (*dStack).pop_back();
  double x = (*dStack).back();
  (*dStack).pop_back();
  if (ListOfObjects->size()>0)
  {
    EpsPolygon* epsp;
    if (ListOfObjects->back()->GetCoordinateList().size() > 0)
    {
      epsp = new EpsPolygon();
      epsp->AddCoordinate(x,y);
      ListOfObjects->push_back(epsp);
    }
    else
    {
      epsp = (*ListOfObjects).back();
      epsp->AddCoordinate(x,y);
    }
  }
}

void EpsCommands::RLineTo(std::vector<EpsPolygon*>* ListOfObjects,std::vector<double>* dStack)
{
  double ry = (*dStack).back();
  (*dStack).pop_back();
  double rx = (*dStack).back();
  (*dStack).pop_back();
  EpsPolygon* epsp = (*ListOfObjects).back();
  epsp->AddCoordinate(epsp->GetCoordinateList().back().at(0)+rx,
                      epsp->GetCoordinateList().back().at(1)+ry
                     );
}

void EpsCommands::ClosePath(std::vector<EpsPolygon*>* ListOfObjects)
{
  EpsPolygon* epsp = (*ListOfObjects).back();
  if (epsp->GetCoordinateList().size() > 0)
    epsp->AddCoordinate(epsp->GetCoordinateList().front().at(0),
                        epsp->GetCoordinateList().front().at(1)
                       );
}

void EpsCommands::SetRGBColor(std::vector<EpsPolygon*>* ListOfObjects,std::vector<double>* dStack)
{
  double b = dStack->back();
  (*dStack).pop_back();
  double g = dStack->back();
  (*dStack).pop_back();
  double r = dStack->back();
  (*dStack).pop_back();
  EpsPolygon* epsp = new EpsPolygon();
  epsp->SetColorRGB(r,g,b);
  ListOfObjects->push_back(epsp);
}

void EpsCommands::SetGray(std::vector<EpsPolygon*>* ListOfObjects,std::vector<double>* dStack)
{
  double gray = dStack->back();
  (*dStack).pop_back();
  EpsPolygon* epsp = new EpsPolygon();
  epsp->SetColorRGB(gray,gray,gray);
  ListOfObjects->push_back(epsp);
}

void EpsCommands::Stroke(std::vector<EpsPolygon*>* ListOfObjects)
{
  (*ListOfObjects).back()->SetFill(false);
}

void EpsCommands::EOFill(std::vector<EpsPolygon*>* ListOfObjects)
{
  ClosePath(ListOfObjects);
  (*ListOfObjects).back()->SetFill(true);
}
